# Copyright (C) 2009 Canonical
#
# Authors:
#  Andrew Higginson
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os
import rsvg
import gtk
import pynotify
import webbrowser
import shutil
import logging
import gettext
import locale
import tempfile
import pickle
import gnomevfs
from random import randrange as rand

from gettext import gettext as _

from SimpleGtkbuilderApp import SimpleGtkbuilderApp

import view.dialogs as dialogs
from utils.motor import ObjectStack
from utils.config import MeMakerConfig
from utils.launchpad import MeMakerLaunchpad
from utils.paths import MeMakerPaths
from view.notebook import NotebookFeatures
from view.notifications import MeMakerNotification
from view.combobox import StoreThemes, ComboboxThemes
from view.libpurple import MeMakerPidginTreeView, PidginParse

from version import VERSION

try:
    from PIL import Image
except ImportError:
    notifier= pynotify.init("MeMaker Modules")
    notifier = pynotify.Notification(("Recommended Module Not Found"), 
                    ("Install 'python-imaging' for better thumbnails."),
	                                 "dialog-info")
    notifier.show()
    from utils.image_loader_dummy import ImageLoader
else:
    from utils.image_loader import ImageLoader
os.chdir(os.sys.path[0])

# Fixme: remove this
AUTHORSFILE = '/usr/share/doc/memaker/AUTHORS'

class MeMakerApp(SimpleGtkbuilderApp):
    
    class featuresObject:
        def __init__(self):
            pass
        
    class runningHeads:
        def __init__(self):
            pass
    
    def __init__(self, datadir, theme_name):
        SimpleGtkbuilderApp.__init__(self, os.path.join(datadir, "ui", 
                                    "memaker.ui"), "memaker")
        self.first = False
        self.datadir = datadir
        self.current_face = "Unset"
        self.features_made = {}
        self.dialogs = dialogs
        self.paths = MeMakerPaths(datadir)
        self.memaker_configuration = MeMakerConfig(self.paths.config_home, 
                                                    self.paths.data_home)
        self.memaker_notifications = MeMakerNotification()
        self.memaker_launchpad = MeMakerLaunchpad()
        self.feature_list = ["head", "hair", "eyes", "mouth", "ears", 
                            "glasses", "nose", "eyebrows", "hat", 
                            "accessories", "beard"]
        self.translation_feature_list = [_("Head"), _("Hair"), _("Eyes"), 
                                    _("Mouth"), _("Ears"), _("Glasses"),
                                    _("Nose"), _("Eyebrows"), _("Hat"),
                                    _("Accessories"), _("Beard")]
        self.clear_it = rsvg.Handle(self.paths.clear_it)
        self.init_translations()
        self.init_launchpad()
        self.notebook_features = NotebookFeatures(self.feature_list, 
                                        self.translation_feature_list, 
                                        datadir)
        self.vbox4.pack_start(self.notebook_features)
        self.connect_iconviews()
        self.connect_buttons()
        self.disable_launchpad_items()
        

        self.store_themes = StoreThemes(self.paths.themes_dir, self.paths.data_home)
        self.combobox_themes = ComboboxThemes(self.store_themes)
        self.alignment_combobox.add(self.combobox_themes)
        self.combobox_themes.connect('changed', self.theme_changed, 
                                      self.store_themes.themes_list_b)
        
        #Set up drag (and drop) for avatarPicture
        targets = [] #?!
        self.drawingarea_avatar.drag_source_set(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
        self.drawingarea_avatar.drag_source_add_uri_targets()
        
        # For each theme start up it's object stack and then assign it 
        # to the the dictionary for reference
        # The reference is the actually the location of the themes.
        self.loaded_heads = {}
        try:
            self.load_pickled_head()
        except Exception:
            logging.info(_("Error found in the pickledHeads file ignoring and moving on..."))
            
        # Now lets check the loaded heads... do we have the same list 
        # as the self.store_themes.themes_list_a
        for items in self.store_themes.themes_list_b:
            if items not in self.loaded_heads.keys():
                print _("Looks like a new theme was added.  We will need to reset everything to adjust.")
                for items in self.store_themes.themes_list_b:
                    self.loaded_heads[items] = ObjectStack()
 
        self.linkbutton_download.hide()
        self.load_last_theme(theme_name, False)
        self.check_no_themes()
        self.progressbar_loading.hide()
            
    # callbacks

    def on_window_main_delete_event(self, widget, event):
        self.window_main.hide()
        while gtk.events_pending():
            gtk.main_iteration()
        self.pickle_face()
        gtk.main_quit()
        
    def on_menuitem_about_activate(self, widget):
		# Parse our AUTHORS file
        file = open(AUTHORSFILE, 'r')
        text = file.read().split('--')
        file.close()
        self.aboutdialog_main.set_authors(text[2].splitlines()[1:])
        self.aboutdialog_main.set_artists(text[4].splitlines()[1:])
        self.aboutdialog_main.set_version(VERSION)
        result = self.aboutdialog_main.run()
        self.aboutdialog_main.hide()
        return result
    
    def on_menuitem_save_as_svg_activate(self, widget):
        save_location = self.save_load_features(gtk.FILE_CHOOSER_ACTION_SAVE,
           "svg", file_ext = {_("Scalable Vector Graphic | SVG"):"svg"})
        self.statusbar.push(1, _("Saving..."))
        try:
            open(save_location, "w").write(self.current_face.printMe())
            self.statusbar.push(1, _("Saved."))
        except Exception, e:
            self.dialogs.error(self.window_main, _("An error occured whilst saving the file."), None, e)
            self.statusbar.push(1, _("An error occured whilst saving."))
            
    def on_menuitem_save_as_png_activate(self, widget):
        pixbuf = self.save_raster_common()
        save_location = self.save_load_features(gtk.FILE_CHOOSER_ACTION_SAVE,
          "png", file_ext = {_("Portable Network Graphic | PNG"):"png"})
        self.statusbar.push(1, _("Saving..."))
        try:
            pixbuf.save(save_location, 'png')
            self.statusbar.push(1, _("Saved."))
        except Exception, e:
            self.dialogs.error(self.window_main, 
                _("An error occured whilst saving the file."), None, e)
            self.statusbar.push(1, _("An error occured whilst saving."))
        
    def on_menuitem_save_as_bmp_activate(self, widget):
        pixbuf = self.save_raster_common()
        save_location = self.save_load_features(gtk.FILE_CHOOSER_ACTION_SAVE,
                     "bmp", file_ext = {_("Bitmap Image | BMP"):"bmp"})
        self.statusbar.push(1, _("Saving..."))
        try:
            pixbuf.save(save_location, 'bmp')
            self.statusbar.push(1, _("Saved."))
        except Exception, e:
            self.dialogs.error(self.window_main, 
                _("An error occured whilst saving the file."), None, e)
            self.statusbar.push(1, _("An error occured whilst saving."))
            
    def on_menuitem_save_as_launchpad_activate(self, widget):
        pixbuf = self.save_raster_common()
        self.statusbar.push(1, _("Uploading new mugshot..."))
        mugshot_old = open(self.paths.old_mugshot_file, 'w')
        mugshot_temporary = tempfile.NamedTemporaryFile()
        try:
            mugshot_old.write(self.memaker_launchpad.get_mugshot(
                                self.paths.config_home, self.paths.cache_home))
            mugshot_old.close()
            pixbuf = pixbuf.scale_simple(192, 192, gtk.gdk.INTERP_BILINEAR)
            pixbuf.save(mugshot_temporary.name, "png")
            self.memaker_launchpad.set_mugshot(self.paths.config_home, 
                                                self.paths.cache_home, 
                                    open(mugshot_temporary.name).read())
            self.statusbar.push(1, _("Done."))
            self.memaker_notifications.notify(_("Mugshot Updated"), 
                    _("Your Launchpad mugshot has been updated."), 
                    mugshot_temporary.name)
        except Exception, e:
            self.dialogs.error(self.window_main, 
                                _("Could not upload new mugshot."), 
                                _("Please check your connection to the internet or try again in a few minutes"), e)
            self.statusbar.push(1, _("An error occured."))
            
    def on_menuitem_save_as_gnome_activate(self, widget):
        pixbuf = self.save_raster_common()                  
        try:
            shutil.move(self.paths.face, self.paths.face_backup)
        except IOError:
            logging.info(_("No Backup"))
        self.statusbar.push(1, _("Saving..."))
        pixbuf = pixbuf.scale_simple(96, 96, gtk.gdk.INTERP_BILINEAR)
        try:
            pixbuf.save(self.paths.face, 'png')
            self.memaker_notifications.notify(_("Profile Updated"), 
                    _("Your GNOME avatar has been updated."), pixbuf)
            self.statusbar.push(1, _("Saved."))
        except IOError, e:
            self.dialogs.error(self.window_main, 
                _("An error occured whilst saving the file."), None, e)
            self.statusbar.push(1, _("An error occured whilst saving."))
            
    def on_menuitem_save_as_pidgin_activate(self, widget):
        self.statusbar.push(1, _("Saving..."))
        treeview = MeMakerPidginTreeView(self.paths.pidgin_dir, self.paths.pidgin_icons_dir)
        dialog = self.dialogs.save_pidgin(self.window_main, 
                _("Please select the account(s) whose avatar(s) you wish to change:"),
                treeview = treeview)
        if dialog[0] == -5:
            try:
                pixbuf = self.save_raster_common()
                pixbuf = pixbuf.scale_simple(96, 96, gtk.gdk.INTERP_BILINEAR)
                save_name = "memaker_%s.png" % rand(0, 9999999)
                save_location = os.path.join(self.paths.pidgin_icons_dir, save_name)
                print save_location
                pixbuf.save(save_location, 'png')
                print "Why didn't it work"
                selected = []
                for ind in dialog[1][1]:
                    selected.append(ind[0])
                pidgin_parse = PidginParse(self.paths.pidgin_dir, self.paths.pidgin_icons_dir)
                pidgin_parse.set_values(selected, save_name)
                self.statusbar.push(1, _("Saved."))
            except Exception, e:
                self.dialogs.error(self.window_main, 
                    _("An error occured whilst saving the file."), None, e)
                self.statusbar.push(1, _("An error occured whilst saving."))

    def on_menuitem_preferences_activate(self, widget):
        if self.memaker_launchpad.check_installed() == True:
            self.label_launchpadlib_status.hide()
        else:
            self.label_launchpadlib_status.show()
            self.vbox_preferences_launchpad.set_sensitive(False)
        result = self.dialog_preferences.run()
        self.dialog_preferences.hide()
        return result
    
    def on_menuitem_restore_launchpad_activate(self, widget):
        old_mugshot = open(self.paths.old_mugshot_file, 'r')
        try:
            self.memaker_launchpad.set_mugshot(self.paths.config_home, 
                                    self.paths.cache_home, old_mugshot.read())
            self.memaker_notifications.notify(_("Mugshot Restored"), 
                        _("Your Launchpad mugshot has been restored."), 
                        self.paths.old_mugshot_file)
            os.remove(self.paths.old_mugshot_file)
        except Exception, e:
            self.dialogs.error(self.window_main, 
                _("Could not contact Launchpad."), 
                _("Please check your connection to the internet or try again in a few minutes"), e)
            
    
    def on_menuitem_quit_activate(self, widget):
        self.window_main.hide()
        while gtk.events_pending():
            gtk.main_iteration()
        self.pickle_face()
        gtk.main_quit()
        
    def on_menuitem_reset_activate(self, widget):
        self.notebook_features.set_current_page(0)
        self.current_face.clearMe()
        self.apply_changes()
        
    def on_linkbutton_download_clicked(self, widget, 
                                    location = "http://memaker.org"):
        webbrowser.open_new(location)
        
    def on_button_remove_clicked(self, widget):
        """
        Remove a feature from the self.current_face objectstack
        """
        self.current_face.deleteFeature(self.get_selected_page())
        self.apply_changes()

    def on_button_up_clicked(self, widget):
        """
        Apply changes after the up-arrow has been clicked
        """
        #Take a snapshot before
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        image1 = svg_handle.get_pixbuf()
        if self.current_face.raiseFeature(self.get_selected_page()):
            self.apply_changes()
            return
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        image2 = svg_handle.get_pixbuf()
        while self.current_face.hasFeature(self.get_selected_page()) and self.not_changed(image1, image2):
            if self.current_face.raiseFeature(self.get_selected_page()):
                break
            svg_handle = rsvg.Handle(data = self.current_face.printMe())
            image2 = svg_handle.get_pixbuf()
        self.apply_changes()

    def on_button_down_clicked(self, widget):
        """
        Apply changes after the down-arrow has been clicked
        """
        #Take a snapshot before
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        image1 = svg_handle.get_pixbuf()
        if self.current_face.lowerFeature(self.get_selected_page()):
            self.apply_changes()
            return
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        image2 = svg_handle.get_pixbuf()
        while self.current_face.hasFeature(self.get_selected_page()) and self.not_changed(image1, image2):
            if self.current_face.lowerFeature(self.get_selected_page()):
                break
            svg_handle = rsvg.Handle(data = self.current_face.printMe())
            image2 = svg_handle.get_pixbuf()
        self.apply_changes()
    
    def on_feature_clicked(self, iconview_list, model_theme):
        """
        Figure out what was clicked and then apply that change to 
        the current face object stack and redraw the drawing area.
        """
        try:
            selected_items = iconview_list.get_selected_items()[0][0]
            location_feature = model_theme[selected_items][0]
        except IndexError:
            return
        self.current_face.addFeature(location_feature, self.get_selected_page())
        iconview_list.unselect_all()
        self.apply_changes()
                   
    def on_dialog_launchpad_authorization_delete_event(self, widget, event):
        self.dialog_launchpad_authorization.hide()
        
    def on_button_authorization_cancel_clicked(self, widget):
        self.dialog_launchpad_authorization.hide()
        
    def on_button_authorization_apply_clicked(self, widget):
        self.dialog_launchpad_authorization.hide()
        
    def on_button_authorization_forward_1_clicked(self, widget):
        """
        Get authorization from the user
        """
        try:
            self.memaker_launchpad.get_auth1()
            self.button_authorization_forward_1.hide()
            self.button_authorization_forward_2.show()
            self.notebook_authorization.next_page()
        except Exception, e:
            self.dialogs.error(self.dialog_launchpad_authorization, 
                _("Could not request an authorization key from Launchpad."), 
                _("Please check your connection to the internet or try again in a few minutes"), e)
            
        
    def on_button_authorization_forward_2_clicked(self, widget):
        """
        Get authorization from Launchpad
        """
        try:
            self.memaker_launchpad.get_auth2(self.paths.launchpad_auth_file)
            self.memaker_configuration.set_key('main', 'launchpadAuth', "True")
            self.button_authorization_apply.set_sensitive(True)
            self.button_authorization_forward_2.set_sensitive(False)
            self.menuitem_save_as_launchpad.set_sensitive(True)
            self.menuitem_save_as_launchpad.set_tooltip_text('')
            self.notebook_authorization.next_page()
        except Exception, e:
            self.dialogs.error(self.dialog_launchpad_authorization, 
                _("Could not convert authorization key into permission from Launchpad."),
                _("Please check your connection to the internet or try again in a few minutes"), e)
        
    def on_button_start_authorization_assistant_clicked(self, widget):
        """
        The routine of getting authentication from Launchpad
        """
        #Reset everything
        self.dialog_preferences.hide()
        self.button_authorization_forward_1.show()
        self.button_authorization_forward_2.hide()
        self.button_authorization_forward_2.set_sensitive(True)
        self.button_authorization_apply.set_sensitive(False)
        self.notebook_authorization.set_current_page(0)
        self.dialog_launchpad_authorization.run()
        
    def on_menuitem_restore_activate(self, widget):
        self.check_backups()
        
    def on_menuitem_restore_gnome_activate(self, widget):   
        shutil.move(self.paths.face_backup, self.paths.face)
        self.memaker_notifications.notify(_("GNOME Avatar Set Back"), 
            _("The change you made to your GNOME Avatar was undone."), 
                self.paths.face)
    
    def on_drawingarea_avatar_drag_begin(self, widget, context, data=None):
        #Generate an icon to represent the image being dragged
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        pixbuf = svg_handle.get_pixbuf()
        #FIXME: There must be some kind of a standard for the dimensions of these previews
        widget.drag_source_set_icon_pixbuf(pixbuf.scale_simple(48, 48, gtk.gdk.INTERP_BILINEAR)) 
        return

    def on_drawingarea_avatar_drag_data_get(self, widget, context, selection, target_type, event_time, data=None):
        uris = list()
        file = open(self.paths.dnd_avatar_svg, "w")
        file.write(self.current_face.printMe())
        uris.append(gnomevfs.get_uri_from_local_path(self.paths.dnd_avatar_svg))
        print uris
        selection.set_uris(uris)
    
    # functions
    
    def pickle_face(self):
        """
        Pickle all the face objects to a file called pickledHeads and quit memaker.
        """
        logging.info(_("Saving your session and quiting..."))
        file_to_unpickle = open(self.paths.pickled_heads_file, "w")
        pickle.dump(self.loaded_heads, file_to_unpickle)
        logging.info(_("Saving the last theme...") + self.last_theme_used)
        self.memaker_configuration.set_key('main', 'lastTheme', 
                                            self.last_theme_used)        
        
    def connect_iconviews(self):
        """
        Connect the feature thumbnails to their corresponding functions
        """
        for feature in self.feature_list:
            tempvar = self.featuresObject()
            tempvar.model = gtk.ListStore(str, gtk.gdk.Pixbuf)
            tempvar.iconview = getattr(self.notebook_features, 
                                        'iconview_%s' % feature)
            tempvar.iconview.set_model(tempvar.model)
            tempvar.iconview.set_pixbuf_column(1)
            tempvar.iconview.connect('selection-changed', 
                                self.on_feature_clicked, tempvar.model)
            self.features_made[feature] = tempvar
        
    def connect_buttons(self):
        """
        Connect the feature buttons to their corresponding functions
        """
        for action in ["up", "down", "remove"]:
            for feature in self.feature_list:
                function_name = getattr(self, "on_button_%s_clicked" % action)
                variable_name = "button_%s_%s" % (action, feature)
                getattr(self.notebook_features, variable_name).connect('clicked', function_name)

    def draw_scene(self, a, v):
        """
        Draw the current face Object stack with cairo
        """
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        cr = self.drawingarea_avatar.window.cairo_create()
        self.clear_it.render_cairo(cr)
        svg_handle.render_cairo(cr)
    
    def disable_launchpad_items(self):
        key = self.memaker_configuration.get_key('main', 'launchpadAuth')
        if key == "False":
            self.menuitem_save_as_launchpad.set_sensitive(False)
            self.menuitem_save_as_launchpad.set_tooltip_text(("You have not yet authorized MeMaker for use with your Launchpad account. Please do so through Edit > Preferences."))
 
    
    def check_backups(self):
        """
        Check if we have backups, if we do offer to restore from them
        """
        backups = {"gnome":".face", "launchpad":"launchpad_mugshot_old.png"}
        for backup_type in backups:
            if os.path.isfile(os.path.join(self.paths.cache_home, backups[backup_type])):
                getattr(self, "menuitem_restore_%s" % backup_type).show()
            else:
                getattr(self, "menuitem_restore_%s" % backup_type).hide()
        if os.path.isfile(os.path.join(self.paths.cache_home, backups["gnome"])) or os.path.isfile(os.path.join(self.paths.cache_home, backups["launchpad"])):
            self.menuitem_restore_none.hide()
        else:
            self.menuitem_restore_none.show()
            
    def theme_changed(self, widget, themes_list_b):
        """
        Restore the proper theme for extraction
        """
        if self.first:
            self.load_features(self.store_themes.themes_list_b[self.combobox_themes.get_active()])
        else:
            self.load_features(self.store_themes.themes_list_b[self.combobox_themes.get_active()], False)
            self.first = True
        

    def load_features(self, theme_location, first=True):
        """
        Load the features from theme_location and place it in the iconview as thumbnails
        """
        self.last_theme_used = theme_location
        self.current_face = self.loaded_heads[theme_location]
        self.apply_changes()
        self.notebook_features.set_current_page(0)
        if first:
            self.progressbar_loading.show()
        self.combobox_themes.set_sensitive(False)
        feat_list_loading = []
        for feat in self.feature_list:
            self.features_made[feat].model.clear()
            feat_list_loading.append({'text':_('Loading %ss ...')% feat, 
                                    'path':theme_location + feat.capitalize()+'/', 
                                    'feature_type':feat})
        #This is going to now show the progress window...
        count = 0.0
        loader_length = len(feat_list_loading) + 1.0
        image_loader = ImageLoader(50, 50, '#000000')
        for feature in feat_list_loading:
            count += 1
            if first:
                self.progressbar_loading.set_text(feature['text'])
                self.progressbar_loading.set_fraction(count/loader_length)
            while gtk.events_pending():
                gtk.main_iteration()
            locations_to_load = [os.path.expanduser(feature['path'])]
            for location_theme in locations_to_load:
                for items in (os.listdir(location_theme)):
                    if os.path.isfile(location_theme+items) == True:
                        pixbuf = image_loader.load_cached_image(location_theme+items)
                        scaled_buf = pixbuf.scale_simple(50, 50, gtk.gdk.INTERP_BILINEAR)
                        picture_location = (location_theme+items)
                        for feat in self.feature_list:
                            if feature['feature_type'] == feat:
                                self.features_made[feat].model.append(
                                                    [picture_location, 
                                                    scaled_buf])
                    else:
                        logging.info(_("Found a folder, but I'm not loading what's inside."))
                    
        #Hide out the loader
        image_loader.close()
        if first:
            self.progressbar_loading.hide()
        self.combobox_themes.set_sensitive(True)
        
    def apply_changes(self):
        """
        Redraw the image to make sure it is all shown on the screen.
        """
        self.drawingarea_avatar.queue_draw()

    def get_selected_page(self):
        """
        Take the current notbook page and return it back to the caller 
        with the name of the notebook
        """
        notebook_index = self.notebook_features.get_current_page()
        return self.feature_list[notebook_index].lower()
        
    def not_changed(self, original_pixbuf, new_pixbuf):
        """
        Takes two pixbufs and returns if they are the exact same or 
        different. True/False
        """
        return original_pixbuf.get_pixels() == new_pixbuf.get_pixels()
        
    def save_load_features(self, dialog_action, file_type, file_ext):
        """
        dialog_action - The open or save mode for the dialog either
        gtk.FILE_CHOOSER_ACTION_OPEN, gtk.FILE_CHOOSER_ACTION_SAVE
            file_name - Default name when doing a save
        """
        file_name = _("MyAvatar")        
        if (dialog_action==gtk.FILE_CHOOSER_ACTION_OPEN):
            dialog_buttons = (gtk.STOCK_CANCEL
                                , gtk.RESPONSE_CANCEL
                                , gtk.STOCK_OPEN
                                , gtk.RESPONSE_OK)
        else:
            dialog_buttons = (gtk.STOCK_CANCEL
                                , gtk.RESPONSE_CANCEL
                                , gtk.STOCK_SAVE
                                , gtk.RESPONSE_OK)

        file_dialog = gtk.FileChooserDialog(title=_("MeMaker - Save Avatar")
                    , action=dialog_action
                    , buttons=dialog_buttons)
        if (dialog_action==gtk.FILE_CHOOSER_ACTION_SAVE):
            file_dialog.set_current_name(file_name+"."+file_type)
            for extension in file_ext.keys():
                filters = gtk.FileFilter()
                filters.set_name(extension)
                filters.add_pattern("*." + file_ext[extension])
                file_dialog.add_filter(filters)
        filters = gtk.FileFilter()
        filters.set_name(_("All files"))
        filters.add_pattern("*")
        file_dialog.add_filter(filters)
        #Path to save the avatar
        file_dialog.set_current_folder(self.paths.home)
        #Init the return value
        result = ""
        if file_dialog.run() == gtk.RESPONSE_OK:
            result = file_dialog.get_filename()
        #Update preferred save location
        self.pref_save_location = file_dialog.get_current_folder()
        file_dialog.destroy()
        #Lets see if the user put an extension on or not.
        for items in file_ext:
            if not result.endswith("."+file_ext[items]):
                result = result+"."+file_ext[items]
            logging.info(_("saving as")), result
            return result
                
    def load_pickled_head(self):
        """
        Restore the pickled file pickledHeads to restore all of the heads or use.
        """
        if os.path.isfile(self.paths.pickled_heads_file):
            file_to_pickle = open(self.paths.pickled_heads_file, "r")
            self.loaded_heads = pickle.load(file_to_pickle)
        else:
            logging.info(_("Previous session not found."))
            
    def init_translations(self):
        gettext.bindtextdomain("memaker", "/usr/share/locale")
        gettext.textdomain("memaker")
        try:
            locale.setlocale(locale.LC_ALL, "")
        except Exception, e:
            logging.exception("setlocale failed: '%s'" % e)
            
    def init_launchpad(self):
        try:
            import LaunchpadIntegration
            LaunchpadIntegration.set_sourcepackagename("memaker")
            LaunchpadIntegration.add_items(self.menu_help, 1, True, False)
        except Exception, e:
            logging.debug("launchpad integration error: '%s'" % e)
         
    def run(self):
        self.window_main.show_all()
        SimpleGtkbuilderApp.run(self)
    
    def save_raster_common(self):
        svg_handle = rsvg.Handle(data = self.current_face.printMe())
        cr = self.drawingarea_avatar.window.cairo_create()
        self.clear_it.render_cairo(cr)
        return svg_handle.get_pixbuf()
        
    def check_no_themes(self):
        # Offer download button if there are no themes
        if len(self.store_themes.themes_list_a) <= 1:
            self.combobox_themes.hide()
            self.linkbutton_download.show()
        else:
            self.linkbutton_download.hide()
            for items in self.store_themes.themes_list_b:
                if self.last_theme_path == items:
                    self.combobox_themes.set_active(self.store_themes.themes_list_b.index(items))
            self.combobox_themes.show()    
            
    def load_last_theme(self, theme_name, first=True):
        if theme_name and os.path.join(self.datadir, "themes", theme_name+"/") in self.loaded_heads:
            self.last_theme_path = os.path.join(self.datadir, "themes", theme_name+"/")
        else:
            self.last_theme_path = self.memaker_configuration.get_key('main', 'lastTheme')
        if not os.path.isdir(self.last_theme_path):
            logging.info(_("Previously used theme was not found.  Was it deleted/moved?"))
            logging.info(_("Falling back to default theme..."))
            self.last_theme_path = os.path.join(self.datadir, "themes", "cocoHead")
            
        if first:
            self.load_features(self.last_theme_path, first)
